/********
	program: 68705R3 simulator
	file: instr.c - instruction table, instruction and interrupt
		simulation.  See instr2.c for individual instruction
		routines.
	Author: Joseph Schachner
	Notes: 
********/

#include <stdio.h>
#include "sim.h"

#ifdef MSC
#include <dos.h>
#endif

extern FILE *logfile;

struct instruction_table_entry {
    PFI function;
    int argument;
    char *name;
    int ncycles;
};

/* The following three items are filled by routines in here and instr2
   and are printed out nicely formatted from sim_instr, below 
*/
char mne[40];
char opcode[40];
char rw[40];

/* Part of this string is used to fill between fields, as needed */
char blanks[]="                                        ";  /* 40 blanks */

char inval_str[]="Invalid op";

invalid( mode)
int mode;
{
    sim_halt();
}

extern int brset(),brclr(),bset(),bclr(),branch();
extern int neg(),com(),lsr(),ror(),asr(),lsl(),rol(),dec(),inc(),tst(),clr();
extern int rti(),rts(),swi(),tax(),clc(),sec(),cli(),sei(),rsp(),nop(),txa();
extern int sub(),cmp(),sbc(),cpx(),and(),bit(),lda(),sta(),eor(),adc(),ora();
extern int add(),jmp(),bsr(),jsr(),ldx(),stx();


struct instruction_table_entry instr_tbl[] =
{
    { brset, 0, "BRSET0", 10 },
    { brclr, 0, "BRCLR0", 10 },
    { brset, 1, "BRSET1", 10 },
    { brclr, 1, "BRCLR1", 10 },
    { brset, 2, "BRSET2", 10 },
    { brclr, 2, "BRCLR2", 10 },
    { brset, 3, "BRSET3", 10 },
    { brclr, 3, "BRCLR3", 10 },
    { brset, 4, "BRSET4", 10 },
    { brclr, 4, "BRCLR4", 10 },
    { brset, 5, "BRSET5", 10 },
    { brclr, 5, "BRCLR5", 10 },
    { brset, 6, "BRSET6", 10 },
    { brclr, 6, "BRCLR6", 10 },
    { brset, 7, "BRSET7", 10 },
    { brclr, 7, "BRCLR7", 10 },
    { bset, 0, "BSET0", 7 },
    { bclr, 0, "BCLR0", 7 },
    { bset, 1, "BSET1", 7 },
    { bclr, 1, "BCLR1", 7 },
    { bset, 2, "BSET2", 7 },
    { bclr, 2, "BCLR2", 7 },
    { bset, 3, "BSET3", 7 },
    { bclr, 3, "BCLR3", 7 },
    { bset, 4, "BSET4", 7 },
    { bclr, 4, "BCLR4", 7 },
    { bset, 5, "BSET5", 7 },
    { bclr, 5, "BCLR5", 7 },
    { bset, 6, "BSET6", 7 },
    { bclr, 6, "BCLR6", 7 },
    { bset, 7, "BSET7", 7 },
    { bclr, 7, "BCLR7", 7 },
    { branch, 0, "BRA", 4 },
    { branch, 1, "BRN", 4 },
    { branch, 2, "BHI", 4 },
    { branch, 3, "BLS", 4 },
    { branch, 4, "BCC", 4 },
    { branch, 5, "BCS", 4 },
    { branch, 6, "BNE", 4 },
    { branch, 7, "BEQ", 4 },
    { branch, 8, "BHCC",4 },
    { branch, 9, "BHCS",4 },
    { branch, 10, "BPL", 4 },
    { branch, 11, "BMI", 4 },
    { branch, 12, "BMC", 4 },
    { branch, 13, "BMS", 4 },
    { branch, 14, "BIL", 4 },
    { branch, 15, "BIH", 4 },
    { neg,  DIRECT, "NEG", 6 },
    { invalid,  DIRECT, inval_str, 0 },
    { invalid,  DIRECT, inval_str, 0 },
    { com,  DIRECT, "COM", 6 },
    { lsr,  DIRECT, "LSR", 6 },
    { invalid,  DIRECT, inval_str, 0 },
    { ror,  DIRECT, "ROR", 6 },
    { asr,  DIRECT, "ASR", 6 },
    { lsl,  DIRECT, "LSL", 6 },
    { rol,  DIRECT, "ROL", 6 },
    { dec,  DIRECT, "DEC", 6 },
    { invalid,  DIRECT, inval_str, 0 },
    { inc,  DIRECT, "INC", 6 },
    { tst,  DIRECT, "TST", 6 },
    { invalid,  DIRECT, inval_str, 0 },
    { clr,  DIRECT, "CLR", 6 },
    { neg,  ACCUM, "NEGA", 4 },
    { invalid,  ACCUM, inval_str, 0 },
    { invalid,  ACCUM, inval_str, 0 },
    { com,  ACCUM, "COMA", 4 },
    { lsr,  ACCUM, "LSRA", 4 },
    { invalid,  ACCUM, inval_str, 0 },
    { ror,  ACCUM, "RORA", 4 },
    { asr,  ACCUM, "ASRA", 4 },
    { lsl,  ACCUM, "LSLA", 4 },
    { rol,  ACCUM, "ROLA", 4 },
    { dec,  ACCUM, "DECA", 4 },
    { invalid,  ACCUM, inval_str, 0 },
    { inc,  ACCUM, "INCA", 4 },
    { tst,  ACCUM, "TSTA", 4 },
    { invalid,  ACCUM, inval_str, 0 },
    { clr,  ACCUM, "CLRA", 4 },
    { neg,  XREG, "NEGX", 4 },
    { invalid,  XREG, inval_str, 0 },
    { invalid,  XREG, inval_str, 0 },
    { com,  XREG, "COMX", 4 },
    { lsr,  XREG, "LSRX", 4 },
    { invalid,  XREG, inval_str, 0 },
    { ror,  XREG, "RORX", 4 },
    { asr,  XREG, "ASRX", 4 },
    { lsl,  XREG, "LSLX", 4 },
    { rol,  XREG, "ROLX", 4 },
    { dec,  XREG, "DECX", 4 },
    { invalid,  XREG, inval_str, 0 },
    { inc,  XREG, "INCX", 4 },
    { tst,  XREG, "TSTX", 4 },
    { invalid,  XREG, inval_str, 0 },
    { clr,  XREG, "CLRX", 4 },
    { neg,  AT_X_BYTE_OFFSET, "NEG", 7 },
    { invalid,  AT_X_BYTE_OFFSET, inval_str, 0 },
    { invalid,  AT_X_BYTE_OFFSET, inval_str, 0 },
    { com,  AT_X_BYTE_OFFSET, "COM", 7 },
    { lsr,  AT_X_BYTE_OFFSET, "LSR", 7 },
    { invalid,  AT_X_BYTE_OFFSET, inval_str, 0 },
    { ror,  AT_X_BYTE_OFFSET, "ROR", 7 },
    { asr,  AT_X_BYTE_OFFSET, "ASR", 7 },
    { lsl,  AT_X_BYTE_OFFSET, "LSL", 7 },
    { rol,  AT_X_BYTE_OFFSET, "ROL", 7 },
    { dec,  AT_X_BYTE_OFFSET, "DEC", 7 },
    { invalid,  AT_X_BYTE_OFFSET, inval_str, 0 },
    { inc,  AT_X_BYTE_OFFSET, "INC", 7 },
    { tst,  AT_X_BYTE_OFFSET, "TST", 7 },
    { invalid,  AT_X_BYTE_OFFSET, inval_str, 0 },
    { clr,  AT_X_BYTE_OFFSET, "CLR", 7},
    { neg,  AT_X, "NEG ,X", 6 },
    { invalid,  AT_X, inval_str, 0 },
    { invalid,  AT_X, inval_str, 0 },
    { com,  AT_X, "COM ,X", 6 },
    { lsr,  AT_X, "LSR ,X", 6 },
    { invalid,  AT_X, inval_str, 0 },
    { ror,  AT_X, "ROR ,X", 6 },
    { asr,  AT_X, "ASR ,X", 6 },
    { lsl,  AT_X, "LSL ,X", 6 },
    { rol,  AT_X, "ROL ,X", 6 },
    { dec,  AT_X, "DEC ,X", 6 },
    { invalid,  AT_X, inval_str, 0 },
    { inc,  AT_X, "INC ,X", 6 },
    { tst,  AT_X, "TST ,X", 6 },
    { invalid,  AT_X, inval_str, 0 },
    { clr,  AT_X, "CLR ,X", 6 },
    { rti,  INH, "RTI", 9 },
    { rts,  INH, "RTS", 6 },
    { invalid,  INH, inval_str, 0 },
    { swi,  INH, "SWI", 11 },
    { invalid,  INH, inval_str, 0 },
    { invalid,  INH, inval_str, 0 },
    { invalid,  INH, inval_str, 0 },
    { invalid,  INH, inval_str, 0 },
    { invalid,  INH, inval_str, 0 },
    { invalid,  INH, inval_str, 0 },
    { invalid,  INH, inval_str, 0 },
    { invalid,  INH, inval_str, 0 },
    { invalid,  INH, inval_str, 0 },
    { invalid,  INH, inval_str, 0 },
    { invalid,  INH, inval_str, 0 },
    { invalid,  INH, inval_str, 0 },
    { invalid,  INH, inval_str, 0 },
    { invalid,  INH, inval_str, 0 },
    { invalid,  INH, inval_str, 0 },
    { invalid,  INH, inval_str, 0 },
    { invalid,  INH, inval_str, 0 },
    { invalid,  INH, inval_str, 0 },
    { invalid,  INH, inval_str, 0 },
    { tax, INH, "TAX", 2 },
    { clc, INH, "CLC", 2 },
    { sec, INH, "SEC", 2 },
    { cli, INH, "CLI", 2 },
    { sei, INH, "SEI", 2 },
    { rsp, INH, "RSP", 2 },
    { nop, INH, "NOP", 2 },
    { invalid,  INH, inval_str, 0 },
    { txa, INH, "TXA", 2 },
    { sub, IMM, "SUB", 2 },
    { cmp, IMM, "CMP", 2 },
    { sbc, IMM, "SBC", 2 },
    { cpx, IMM, "CPX", 2 },
    { and, IMM, "AND", 2 },
    { bit, IMM, "BIT", 2 },
    { lda, IMM, "LDA", 2 },
    { invalid,  IMM, inval_str, 0 },
    { eor, IMM, "EOR", 2 },
    { adc, IMM, "ADC", 2 },
    { ora, IMM, "ORA", 2 },
    { add, IMM, "ADD", 2 },
    { invalid,  IMM, inval_str, 0 },
    { bsr, REL, "BSR", 8 },
    { ldx, IMM, "LDX", 2 },
    { invalid,  IMM, inval_str, 0 },
    { sub, DIRECT, "SUB", 4 },
    { cmp, DIRECT, "CMP", 4 },
    { sbc, DIRECT, "SBC", 4 },
    { cpx, DIRECT, "CPX", 4 },
    { and, DIRECT, "AND", 4 },
    { bit, DIRECT, "BIT", 4 },
    { lda, DIRECT, "LDA", 4 },
    { sta, DIRECT, "STA", 5 },
    { eor, DIRECT, "EOR", 4 },
    { adc, DIRECT, "ADC", 4 },
    { ora, DIRECT, "ORA", 4 },
    { add, DIRECT, "ADD", 4 },
    { jmp, DIRECT, "JMP", 3 },
    { jsr, DIRECT, "JSR", 7 },
    { ldx, DIRECT, "LDX", 4 },
    { stx, DIRECT, "STX", 5 },
    { sub, EXT, "SUB", 5 },
    { cmp, EXT, "CMP", 5 },
    { sbc, EXT, "SBC", 5 },
    { cpx, EXT, "CPX", 5 },
    { and, EXT, "AND", 5 },
    { bit, EXT, "BIT", 5 },
    { lda, EXT, "LDA", 5 },
    { sta, EXT, "STA", 6 },
    { eor, EXT, "EOR", 5 },
    { adc, EXT, "ADC", 5 },
    { ora, EXT, "ORA", 5 },
    { add, EXT, "ADD", 5 },
    { jmp, EXT, "JMP", 4 },
    { jsr, EXT, "JSR", 6 },
    { ldx, EXT, "LDX", 5 },
    { stx, EXT, "STX", 6 },
    { sub, AT_X_WORD_OFFSET, "SUB", 6 },
    { cmp, AT_X_WORD_OFFSET, "CMP", 6 },
    { sbc, AT_X_WORD_OFFSET, "SBC", 6 },
    { cpx, AT_X_WORD_OFFSET, "CPX", 6 },
    { and, AT_X_WORD_OFFSET, "AND", 6 },
    { bit, AT_X_WORD_OFFSET, "BIT", 6 },
    { lda, AT_X_WORD_OFFSET, "LDA", 6 },
    { sta, AT_X_WORD_OFFSET, "STA", 7 },
    { eor, AT_X_WORD_OFFSET, "EOR", 6 },
    { adc, AT_X_WORD_OFFSET, "ADC", 6 },
    { ora, AT_X_WORD_OFFSET, "ORA", 6 },
    { add, AT_X_WORD_OFFSET, "ADD", 6 },
    { jmp, AT_X_WORD_OFFSET, "JMP", 5 },
    { jsr, AT_X_WORD_OFFSET, "JSR", 9 },
    { ldx, AT_X_WORD_OFFSET, "LDX", 6 },
    { stx, AT_X_WORD_OFFSET, "STX", 7 },
    { sub, AT_X_BYTE_OFFSET, "SUB", 5 },
    { cmp, AT_X_BYTE_OFFSET, "CMP", 5 },
    { sbc, AT_X_BYTE_OFFSET, "SBC", 5 },
    { cpx, AT_X_BYTE_OFFSET, "CPX", 5 },
    { and, AT_X_BYTE_OFFSET, "AND", 5 },
    { bit, AT_X_BYTE_OFFSET, "BIT", 5 },
    { lda, AT_X_BYTE_OFFSET, "LDA", 5 },
    { sta, AT_X_BYTE_OFFSET, "STA", 6 },
    { eor, AT_X_BYTE_OFFSET, "EOR", 5 },
    { adc, AT_X_BYTE_OFFSET, "ADC", 5 },
    { ora, AT_X_BYTE_OFFSET, "ORA", 5 },
    { add, AT_X_BYTE_OFFSET, "ADD", 5 },
    { jmp, AT_X_BYTE_OFFSET, "JMP", 4 },
    { jsr, AT_X_BYTE_OFFSET, "JSR", 8 },
    { ldx, AT_X_BYTE_OFFSET, "LDX", 5 },
    { stx, AT_X_BYTE_OFFSET, "STX", 6 },
    { sub, AT_X, "SUB ,X", 4 },
    { cmp, AT_X, "CMP ,X", 4 },
    { sbc, AT_X, "SBX ,X", 4 },
    { cpx, AT_X, "CPX ,X", 4 },
    { and, AT_X, "AND ,X", 4 },
    { bit, AT_X, "BIT ,X", 4 },
    { lda, AT_X, "LDA ,X", 4 },
    { sta, AT_X, "STA ,X", 5 },
    { eor, AT_X, "EOR ,X", 4 },
    { adc, AT_X, "ADC ,X", 4 },
    { ora, AT_X, "ORA ,X", 4 },
    { add, AT_X, "ADD ,X", 4 },
    { jmp, AT_X, "JMP ,X", 3 },
    { jsr, AT_X, "JSR ,X", 7 },
    { ldx, AT_X, "LDX ,X", 4 },
    { stx, AT_X, "STX ,X", 5 }
};

struct program_model pgm_model;

/**********************************************************************/
int sim_reset()
{
    /* See figure 7-7 in 6805 tech data for reset, intr handling desc */

    pgm_model.ccr = CC_I;
    rsp( INH);          /* reset stack pointer to $7F */
    sim_write( PORT_A_DDR, 0 );
    sim_write( PORT_B_DDR, 0 );
    sim_write( PORT_C_DDR,0  );
    external_intr = 0;   /* clear INT request latch */
    sim_write( TIMER_CONTROL, 0x7F);
    sim_write( MISC_REGISTER, 0xFF);
    sim_timer_init();   /* prescaler and ctr to all 1, set TCR bit */
                        /* see 5-8 , 6805 tech data, for more. */
    
    /* Load options from MOR into control logic */
    /* partially done by sim_timer_init, above */

    pgm_model.pc = (sim_read( 0xFFE) << 8) + sim_read( 0xFFF);    
    cycle_count = 0;
}
/**********************************************************************/
int get_ea( addr_mode )
/* Called by most routines in instr2.c.  Returns the address from which
   data is to be fetched (or to which to write), based on address mode
   and an appropriate number of bytes at the current pc, which is assumed
   to point to the byte after the instruction.
   (Exception: bit-test-and-branch instructions call this routine twice:
   the first time mode is DIRECT to get the address of the byte to test,
   and the second time mode is BTB to get the address to which to branch.)
*/
{
    int retval, i, j;
    char buff[40];

    switch (addr_mode)
    {
    case ACCUM:
	retval = RW_A;
	break;
    case XREG:
	retval = RW_X;
	break;
    case AT_X:
	retval = pgm_model.x;
	break;
    case AT_X_BYTE_OFFSET:
	retval = sim_read( pgm_model.pc);
	if (display_enabled)
	{
            sprintf( buff, "   $%x,X", retval);
            strcat( mne, buff);
	    sprintf( buff, " %02x", retval);
	    strcat( opcode, buff);
        }
	retval += pgm_model.x;
	pgm_model.pc++;
	break;
    case AT_X_WORD_OFFSET:
	i = sim_read( pgm_model.pc);
	pgm_model.pc++;
	j = sim_read( pgm_model.pc);
	pgm_model.pc++;
        retval = i * 256 + j;
        if (display_enabled)
        {
            sprintf( buff, "   $%x,X", retval);
            strcat( mne, buff);
	    sprintf( buff, " %02x %02x", i, j);
	    strcat( opcode, buff);
        }
	retval += pgm_model.x;
	break;
    case IMM:
	i = sim_read( pgm_model.pc);
	retval = pgm_model.pc;
	pgm_model.pc++;
        if (display_enabled)
        {
	    sprintf( buff, "   #$%02x", i);
            strcat( mne, buff);
	    sprintf( buff, " %02x", i);
	    strcat( opcode, buff);
 	}
	break;	
    case DIRECT:
	retval = sim_read( pgm_model.pc);
	pgm_model.pc++;
	if (display_enabled)
	{
	    sprintf( buff, "   $%02x", retval);
            strcat( mne, buff);
	    sprintf( buff, " %02x", retval);
	    strcat( opcode, buff );
	}
	break;
    case EXT:
	i = sim_read( pgm_model.pc);
	pgm_model.pc++;
	j = sim_read( pgm_model.pc);
	pgm_model.pc++;
        retval = i * 256 + j;
        if (display_enabled)
        {
            sprintf( buff, "   $%03x", retval);
            strcat( mne, buff);
	    sprintf( buff, " %02x %02x", i, j);
	    strcat( opcode, buff);
        }
	break;
    case BTB: /* second of two bytes for BTB - first is DIRECT, second REL */
	j = i = sim_read( pgm_model.pc);
	pgm_model.pc++;
	if (i > 127)
	    i -= 256;
	retval = pgm_model.pc + i;
	if (display_enabled)
	{
	    sprintf( buff, ",$%03x", retval);
            strcat( mne, buff);
	    sprintf( buff, " %02x", j);
	    strcat( opcode, buff );  /* skip 2 of 3 leading spaces */
	}
	break;
    case REL:
	j = i = sim_read( pgm_model.pc);
	pgm_model.pc++;
	if (i > 127)
	    i -= 256;
	retval = pgm_model.pc + i;
	if (display_enabled)
	{
	    sprintf( buff, "   $%03x", retval);
            strcat( mne, buff);
	    sprintf( buff, " %02x", j);
	    strcat( opcode, buff );  /* skip 2 of 3 leading spaces */
	}
	break;
    default:
        retval = -1;
    }
    return( retval);
}
/**********************************************************************/
int sim_instr()
/* Called from instr_or_intr, below, to do instruction at pc */
/* Also called from sim_r3.c for disassembly */
{
    int iopcode, i, j;

    iopcode = sim_read( pgm_model.pc );
    sprintf( opcode, "%03x  %02x", pgm_model.pc, iopcode);
    pgm_model.pc++;
    strcpy( mne, instr_tbl[iopcode].name);
    rw[0] = '\0';    /* empty string */
    if (!disassemble)
	sim_timer_update( instr_tbl[iopcode].ncycles );    

    (*instr_tbl[iopcode].function)( instr_tbl[iopcode].argument);

    if (!disassemble)
	cycle_count += instr_tbl[iopcode].ncycles;
    if (display_enabled)
    {
	settextposition( 7, 1);
	delete_line(PROMPTLINE+2);
	settextposition( 24, 1);
	printf( "%s", opcode );
	settextposition( 24, 19);
	printf( "%s", mne);
	if (!disassemble)
	{
	    settextposition( 24, 40);
	    printf( "%s", rw); /* nothing in rw if disassemble, see append_rw*/
	    settextposition( 24, 70);
	    printf( "%9ld", cycle_count);
	}
	settextposition( PROMPTLINE, 1);  /* return cursor to default pos */
	
	if (log_enabled)       /* Note: never true if disassemble */
	{
	    fprintf( logfile, "%s", opcode);
            i = strlen( opcode);
	    fprintf( logfile, "%s", &blanks[40-19+i]);
	    fprintf( logfile, "%s", mne );
	    i = strlen( mne);
	    fprintf( logfile, "%s", &blanks[40-(40-19)+i]);
	    fprintf( logfile, "%s", rw);
	    i = strlen( rw);
	    fprintf( logfile, "%s", &blanks[40-(70-40)+i]);
	    fprintf( logfile, "%9ld\n", cycle_count);
	}
    }
}
/**********************************************************************/
int pull()
/* The 6805 has no user "pull" instruction.  That's why this routine is
   here, instead of in instr2.c.  This routine used by rts, rti in instr2
*/
{
    if (pgm_model.sp < 0x7F)
	pgm_model.sp++;
    else
	pgm_model.sp = 0x61;
    return( sim_read( pgm_model.sp));
}
/**********************************************************************/
int push( value)
int value;
/* The 6805 series has no user "push" instruction.  That's why this routine
   is here, instead of in instr2.c.  This routine used by 'interrupt', below
*/
{
    sim_write( pgm_model.sp, value);
    if (pgm_model.sp > 0x61)
	pgm_model.sp--;
    else
	pgm_model.sp = 0x7F; 
}
/**********************************************************************/
int rinterrupt( vector_addr)
/* Stack registers, fetch vector, and display a message */
{
    push( pgm_model.pc & 0xFF);				/* PCL */
    push( ((pgm_model.pc & 0xF00) >> 8) | 0xF0 );	/* PCH */
    push( pgm_model.x);					/*  X  */
    push( pgm_model.a);					/*  A  */
    push( pgm_model.ccr | 0xE0 );			/* CCR */
    pgm_model.ccr |= CC_I;   /* Mask interrupts */
    pgm_model.pc = sim_read( vector_addr)  * 256;
    pgm_model.pc += sim_read( vector_addr + 1 ) ;
    pgm_model.pc &= 4095;
    if (display_enabled)
    {
	settextposition( 7, 1);
	delete_line(PROMPTLINE+2);
	settextposition( 24, 1);
	if (vector_addr == 0xFFC)
	   printf( "==> Interrupt service, SWI");
	else if (vector_addr == 0xFFA)
	   printf( "==> Interrupt service, INT");
	else if (vector_addr == 0xFF8)
	   printf( "==> Interrupt service, INT2 or TIMER");
	if (pgm_model.sp > 0x7A)
	   printf("   *** WARNING: Stack overflow");
	if (log_enabled)
	{
	    if (vector_addr == 0xFFC)
	        fprintf(logfile, "==> Interrupt service, SWI");
	    else if (vector_addr == 0xFFA)
	        fprintf(logfile, "==> Interrupt service, INT");
	    else if (vector_addr == 0xFF8)
	        fprintf(logfile, "==> Interrupt service, INT2 or TIMER");
	    if (pgm_model.sp > 0x7A)
	        fprintf(logfile,"   *** WARNING: Stack overflow\n");
	    else
		fprintf(logfile, "\n");
	}
    }
}
/**********************************************************************/
int instr_or_intr()
/* called from main to do one instruction */
{
    int i,j;

    if (!(pgm_model.ccr & CC_I))
    {
	if (external_intr)
	{
	    external_intr = 0;   /* clear INT request latch */
	    rinterrupt( 0xFFA);   /* vector address = INT */
	}
	else
	{
	    i = sim_read( TIMER_CONTROL);
	    j = sim_read( MISC_REGISTER);
	    if ( (( i & 0xC0) == 0x80) || ((j & 0xC0) == 0x80) )
		rinterrupt( 0xFF8); /* vector address = timer or INT2 */
	}
    }
    sim_instr();    /* SWI is handled as a regular instruction */
}

/**********************************************************************/
append_rw( readflag, addr, data)
int readflag;   /* 1 = Read, 0 = Write */
int addr, data;
/* Used by routines in instr2.c to show bytes read, written */
{
    char buff[50];
    if (display_enabled && !disassemble)
    {
	if (addr >= 0)
	{
	    if (readflag)
		sprintf( buff, "%03x=>%02x ", addr, data);
	    else
		sprintf( buff, "%03x<=%02x ", addr, data);
	}
	else
	{
	    char rname[20];

	    if (addr == RW_A)
		strcpy( rname, "ACCUM");
	    else if (addr == RW_X)
		strcpy( rname, "X_REG");
	    else if (addr == RW_SP)
		strcpy( rname, "SP");
	    if (readflag)
		sprintf( buff, "%s=>%02x ", rname, data);
	    else
		sprintf( buff, "%s<=%02x ", rname, data);
		
	}
	strcat( rw, buff);
    }
}



delete_line( lineno)
int lineno;
{
#ifdef MSC

    union REGS regs;

    lineno--;            /* DOS BIOS counts lines from 0, not 1 */
    regs.h.ah = 6;       /* Service 6 = scroll text window up */
    regs.h.al = 1;       /* Number of lines to scroll up */
    regs.h.ch = lineno;  /* row# of up left */
    regs.h.cl = 0;       /* column# of up left */
    regs.h.dh = 24;      /* row# of low right */
    regs.h.dl = 79;      /* column# of low right */
    regs.h.bh = 7;       /* foreground white, background black, no blink */
    int86( 16, &regs, &regs);
#endif
#ifdef VT52
    settextposition( lineno, 1);
    printf("\033M");    /* escape M = delete line the cursor is on */
			/* the rest moves up one line, last line is free */
#endif
}

/************************ end of file instr.c ***************************/
